package sample;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.math.BigInteger;
import java.net.URL;
import java.util.Scanner;
import com.factset.rt.RT_Consumer;

import javax.net.ssl.HttpsURLConnection;



public class BootstrapUtility {
    static final int NUM_ARGS = 2;

    public static void usage() {
        System.err.println("Usage: java BootstrapUtility <username> <serial>");
    }

    public static void main(String[] args) {
        if(args.length < NUM_ARGS) {
            System.err.println("Not enough arguments");
            usage();
            return;
        }
        RT_Consumer consumer = new RT_Consumer("");
        try {
			consumer.loadConfig("etc/fdsrt.cfg");
		} catch (IOException e) {
			System.out.println("Unable to load config file: " + e.getMessage());
			return;
		}
        String user = args[0];
        String serial = args[1];
        System.out.println("Requesting new passcode");
        try {
			consumer.requestPasscode(user, serial);
		} catch (IOException e) {
            System.out.println("Error: " + e.getMessage());
            return;
		}

        // Wait for the user to enter the passcode successfully
        Scanner input = new Scanner(System.in);
        String passcode;
        RT_Consumer.KeyCounter keyCounter;
        while(true) {
            System.out.print("\nEnter passcode sent via email or exit to quit: ");
            passcode = input.nextLine();
            if(passcode.equals("exit")) {
                return;
            }
            try {
                keyCounter = consumer.getKeyAndCounter(user, serial, passcode);
                break;
            } catch (IOException e) {
                System.out.println("Error: " + e.getMessage());
                System.out.println("Please ensure you typed the passcode properly");
            }
        }
        System.out.println("Success");
        System.out.println("Key: " + keyCounter.key);
        System.out.println("Counter: " + keyCounter.counter);
        // Write key/counter to file named <user-serial>.data containing key and counter for use with datafeed toolkit
        try {
            String filename = user+"-"+serial+".data";
            File file = new File(filename);
            PrintWriter writer = new PrintWriter(file);
            writer.println(keyCounter.key);
            writer.println(keyCounter.counter);
            writer.close();
            System.out.println("Wrote key and counter to " + file.getCanonicalPath());
        } catch (IOException e) {
            System.err.println("Error: " + e.getMessage());
            return;
        }
    }
}