package sample;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.UnknownHostException;
import java.net.URL;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

import javax.security.auth.login.LoginException;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.DefaultTableCellRenderer;


import org.xml.sax.SAXException;

import com.factset.rt.ControlHandler;
import com.factset.rt.MessageHandler;
import com.factset.rt.NotifyHandler;
import com.factset.rt.RT_Consumer;
import com.factset.rt.RT_Consumer.Subscription;
import com.factset.rt.RT_FidField;
import com.factset.rt.RT_Message;
import com.factset.rt.RT_Request;
import com.factset.rt.RT_Level2Feature;
import com.factset.rt.FIDS;

public class SwingL2 implements MessageHandler, ControlHandler, NotifyHandler {
	
	private JFrame frame;
	private JTabbedPane uriTabPane;
	private JComboBox enterpriseUriCombo;
	private JComboBox workstationUriCombo;
	private JComboBox serviceCombo;
	private JComboBox keyCombo;
	private JButton connectButton;
	private JButton disconnectButton;
	private JButton requestButton;
	private JButton cancelButton;
	private JCheckBox totalViewCheck;
	private JCheckBox otpCheck;
	private JButton closeButton;
	private JLabel statusLabel;
	private JLabel updatesLabel;
	private SwingL2Table bidTable;
	private SwingL2Table askTable;
	
	private RT_Consumer consumer;
	private Subscription requestedSub = null;
	private final Logger logger = Logger.getLogger("SwingL2");
	private final Preferences prefs = Preferences.userNodeForPackage(getClass());
	private int numUpdates = 0;

	float first_bid, first_ask;
	private final ReentrantLock _mutex = new ReentrantLock(false);

	private void msgbox(String s)	{
   		JOptionPane.showMessageDialog(null, s);
	}
	
	private final Runnable dispatchRunnable = new Runnable() {
		@Override
		public void run() {
			consumer.dispatch();
		}
	};
	
	@Override
	public void onNotify() {
		SwingUtilities.invokeLater(dispatchRunnable);
	}

	class BidCellRenderer extends DefaultTableCellRenderer {
	
   		private final TableCellRenderer defaultTableCellRenderer= new JTable().getDefaultRenderer(Object.class);
    	
		@Override
    	public Component getTableCellRendererComponent( JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) 
		{
			Component c = defaultTableCellRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
			try {
				_mutex.lock();
				try {
					float curr_bid = Float.valueOf((String) table.getValueAt(row, 0));
					float f_ask = Float.valueOf((String) askTable.getValueAt(0, 0));
					if (curr_bid > first_ask && curr_bid > f_ask)
						c.setBackground(Color.RED);
					else
						c.setBackground(Color.WHITE);
				}
				catch(Exception ex) {
					logger.warning(ex.getMessage());
				}
				_mutex.unlock();
			}
			catch(NumberFormatException e) {
				logger.log(Level.WARNING, "Exception while converting bid row index:", e);
			}
        	return c;
    	}
	}

	class AskCellRenderer extends DefaultTableCellRenderer {
	
   		private final TableCellRenderer defaultTableCellRenderer= new JTable().getDefaultRenderer(Object.class);
    	
		@Override
    	public Component getTableCellRendererComponent( JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) 
		{
        	Component c = defaultTableCellRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
			try {
				_mutex.lock();
				float curr_ask = Float.valueOf((String) table.getValueAt(row, 0));
				float f_bid = Float.valueOf((String) bidTable.getValueAt(0, 0));
				if (curr_ask < first_bid && curr_ask < f_bid)
					c.setBackground(Color.RED);
				else
					c.setBackground(Color.WHITE);
				_mutex.unlock();
			}
			catch(NumberFormatException e) {
				logger.log(Level.WARNING, "Exception while converting ask row index:", e);
			}
        	return c;
    	}
	}
	
	@Override
	public void onMessage(Subscription sub, RT_Message msg) {
		if (msg.isError()) {
			sub.cancel();
			cancelButton.setEnabled(false);
			requestedSub = null;
			numUpdates = 0;
			
			updatesLabel.setText("Stream Closed: " + msg.getErrorDescription());
			bidTable.setRowCount(0);
			askTable.setRowCount(0);
		}
		else if (msg.isStale()) {
			updatesLabel.setText("Stream Stale: " + msg.getStaleReason());
			bidTable.setRowCount(0);
			askTable.setRowCount(0);
		}
		else {
			updatesLabel.setText("Stream OK, NumUpdates: " + numUpdates);
			
			if (msg.isResponse()) {
				statusLabel.setText("Key: " + msg.getKey() + "  NumFids: " +
						msg.count() + "  PermID: " + msg.getPermissions());
			}
			
			if (!msg.getField(FIDS.BID_INDEX_1).isEmpty()) {
				int idx = Integer.valueOf(msg.getField(FIDS.BID_INDEX_1));
				if (msg.getField(FIDS.MSG_TYPE).equals("D")) {
						bidTable.removeRow(idx);
						try {
							if (idx == 0) {
								first_bid = Float.valueOf((String) bidTable.getValueAt(0,0));
							}
						}
						catch(Exception ex) {
							logger.warning(ex.getMessage());
						}
				}
				else {
					String orderCode = msg.getField(FIDS.ORDER_CODE);
					for (int i = 0; i < bidTable.getRowCount(); ++i) {
						if (bidTable.getValueAt(i, 3).equals(orderCode)) {
							bidTable.removeRow(i);
							break;
						}
					}

					bidTable.insertRow(idx, msg.getField(FIDS.BID_1), msg.getField(FIDS.BID_VOL_1), msg.getField(FIDS.BID_TIME_1), msg.getField(FIDS.ORDER_CODE), msg.getField(FIDS.MARKET_MAKER_ID));
					if (idx == 0) {
						first_bid = Float.valueOf( msg.getField(FIDS.BID_1));
					}
				}
			}
			if (!msg.getField(FIDS.ASK_INDEX_1).isEmpty()) {
				int idx = Integer.valueOf(msg.getField(FIDS.ASK_INDEX_1));
				if (msg.getField(FIDS.MSG_TYPE).equals("D")) {
					askTable.removeRow(idx);
					try {
						if (idx == 0) {
							first_ask = Float.valueOf((String) askTable.getValueAt(0,0));
						}
					}
					catch(Exception ex) {
						logger.warning(ex.getMessage());
					}
				}
				else {
					String orderCode = msg.getField(FIDS.ORDER_CODE);
					for (int i = 0; i < askTable.getRowCount(); ++i) {
						if (askTable.getValueAt(i, 3).equals(orderCode)) {
							askTable.removeRow(i);
							break;
						}
					}
					askTable.insertRow(idx, msg.getField(FIDS.ASK_1), msg.getField(FIDS.ASK_VOL_1), msg.getField(FIDS.ASK_TIME_1), msg.getField(FIDS.ORDER_CODE), msg.getField(FIDS.MARKET_MAKER_ID));
					if (idx == 0) {
						first_ask = Float.valueOf( msg.getField(FIDS.ASK_1));
					}
				}
			}
			++numUpdates;
		}
	}
	
	@Override
	public void onControl(boolean isConnected, RT_Message controlMsg) {
		if (controlMsg.getKey().equals("CONNECTED")) {
			String[] services = consumer.getServices();
			for (int i=0; i<services.length; ++i)
				serviceCombo.addItem(services[i]);
			
			requestButton.setEnabled(true);
			statusLabel.setText("Connected to: " +
					(String)((JComboBox)uriTabPane.getSelectedComponent()).getSelectedItem());
		}
		else if (controlMsg.getKey().equals("DISCONNECTED")) {
			serviceCombo.removeAllItems();
			requestButton.setEnabled(false);
			statusLabel.setText("Lost connection... Retrying");
		}
		else if (controlMsg.getKey().equals("SERVICE_ENABLE")) {
			for (RT_FidField field : controlMsg) {
				serviceCombo.addItem(field.getValue());
			}
		}
		else if (controlMsg.getKey().equals("SERVICE_DISCONNECT")) {
			for (RT_FidField field : controlMsg) {
				serviceCombo.removeItem(field.getValue());
			}
		}
		serviceCombo.setSelectedItem(prefs.get("SERVICE", ""));
	}
	
	private void connect() {
		String uri =
				(String)((JComboBox)uriTabPane.getSelectedComponent()).getSelectedItem();
		if (uri == null)
			uri = "";

		try {
			if (uriTabPane.getSelectedComponent() == enterpriseUriCombo)
				if(otpCheck.isSelected()) {
					JOptionPane.showMessageDialog(frame, "Please select a file named <keyId>.data and containing <your_key> on the first line and <your_counter> in decimal format on the second.", "Select Key/Counter File", JOptionPane.INFORMATION_MESSAGE);
					JFileChooser chooser = new JFileChooser();
					FileNameExtensionFilter filter = new FileNameExtensionFilter("Data Files (*.data)", "data");
					chooser.setFileFilter(filter);
					chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
					if(chooser.showOpenDialog(frame)!=JFileChooser.APPROVE_OPTION) {
						return;
					}
					String fullPath = chooser.getSelectedFile().getAbsolutePath();
					String path = "";
					String deviceId = "";
					int sepLoc = fullPath.lastIndexOf("\\");
					if(sepLoc!=-1) {
						path = fullPath.substring(0, sepLoc);
						deviceId = fullPath.substring(sepLoc+1);
					} else {
						deviceId = fullPath;
					}
					int dotLoc = deviceId.lastIndexOf(".");
					if(dotLoc!=-1) {
						deviceId = deviceId.substring(0, dotLoc);
					}
					String user = "";
					String host = "";
					int atLoc = uri.indexOf("@");
					if(atLoc!=-1) {
						user = uri.substring(0, atLoc);
						host = uri.substring(atLoc+1);
					}
					int colLoc = user.indexOf(":");
					if(colLoc!=-1) {
						user = user.substring(0, colLoc);
					}
					if(host.isEmpty() || user.isEmpty()) {
						JOptionPane.showMessageDialog(frame, "Host or user cannot be discerned. Ensure connection uri is in the form <user>-<serial>@<host>", "Error", JOptionPane.ERROR_MESSAGE);
						return;
					}
					if(deviceId.isEmpty()) {
						JOptionPane.showMessageDialog(frame, "Error determining keyId from file. Ensure key/counter file is named <keyId>.data", "Error", JOptionPane.ERROR_MESSAGE);
						return;
					}
					consumer = new RT_Consumer(host, user, deviceId, null, null, path, false);
				} else {
					consumer = new RT_Consumer(uri);
				}
			else {
				String user;
				String pass = "";
				int delimIdx = uri.indexOf(':');
				if (delimIdx == -1)
					user = uri;
				else {
					user = uri.substring(0, delimIdx);
					pass = uri.substring(delimIdx+1);
				}
				consumer = RT_Consumer.newWorkstationInstance(user, pass);
			}
			consumer.setNotifyHandler(this);
			consumer.setControlHandler(this);
			logger.info("Connecting to " + uri);
			consumer.connect();
		} catch (UnknownHostException e) {
			String errMsg;
			if (uriTabPane.getSelectedComponent() == enterpriseUriCombo)
				errMsg = "Error resolving host from the connection string.";
			else
				errMsg = "No running workstation detected.";
				
			JOptionPane.showMessageDialog(frame, errMsg, "Error",
					JOptionPane.ERROR_MESSAGE);
			return;
		} catch (IOException e) {
			logger.severe("Error connecting to DataFeed server: " +
					e.getMessage());
			JOptionPane.showMessageDialog(frame, "Error connecting:\n" +
					e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			return;
		} catch (LoginException e) {
			JOptionPane.showMessageDialog(frame,
					"Incorrect username or password.", "Error",
					JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		addItemToCombo((JComboBox)uriTabPane.getSelectedComponent(), uri);
		uriTabPane.getSelectedComponent().setEnabled(false);
		uriTabPane.setEnabledAt((uriTabPane.getSelectedIndex() == 0 ? 1 : 0), false);
		
		for (String service : consumer.getServices())
			serviceCombo.addItem(service);
		serviceCombo.setSelectedItem(prefs.get("SERVICE", ""));
		
		connectButton.setEnabled(false);
		disconnectButton.setEnabled(true);
		requestButton.setEnabled(true);
		frame.getRootPane().setDefaultButton(requestButton);
		
		statusLabel.setText("Connected to: " + uri);
		consumer.enable(new RT_Level2Feature());
	}
	
	private void disconnect() {
		consumer.disconnect();
		
		resetGUIState();
		requestedSub = null;
		numUpdates = 0;
	}
	
	private void request() {
		cancel();
		String service = (String)serviceCombo.getSelectedItem();
		String key = (String)keyCombo.getSelectedItem();
		if (key == null)
			key = "";
		
		addItemToCombo(keyCombo, key);
		prefs.put("SERVICE", service);
		
		key += ":L2";
		if (totalViewCheck.isSelected()) {
			key += ":TV";
		}
		RT_Request req = new RT_Request(service, key);		
		requestedSub = consumer.request(req, this);
		
		cancelButton.setEnabled(true);
		bidTable.setRowCount(0);
		askTable.setRowCount(0);
		statusLabel.setText("Request sent for " + service + "::" + key);
	}
	
	private void cancel() {
		if (requestedSub != null) {
			requestedSub.cancel();
			requestedSub = null;
			
			cancelButton.setEnabled(false);
			numUpdates = 0;
			
			statusLabel.setText("Cancelled subscription");
			updatesLabel.setText("NumUpdates: 0");
		}
	}
	
	private void resetGUIState() {
		serviceCombo.removeAllItems();
		uriTabPane.getSelectedComponent().setEnabled(true);
		uriTabPane.setEnabled(true);
		uriTabPane.setEnabledAt((uriTabPane.getSelectedIndex() == 0 ? 1 : 0), true);
		
		connectButton.setEnabled(true);
		disconnectButton.setEnabled(false);
		requestButton.setEnabled(false);
		cancelButton.setEnabled(false);
		
		statusLabel.setText("Disconnected");
		updatesLabel.setText("NumUpdates: 0");
	}
	
	private void addComponentsToPane() {
		Container pane = frame.getContentPane();
		((JComponent)pane).setBorder(BorderFactory.createEmptyBorder(0, 10,
				                                                     10, 10));
		pane.setLayout(new GridBagLayout());
		
        enterpriseUriCombo = new JComboBox();
        enterpriseUriCombo.setPreferredSize(new Dimension(225, 25));
        enterpriseUriCombo.setMinimumSize(new Dimension(225, 25));
        enterpriseUriCombo.setEditable(true);
        getItemsFromPrefs(enterpriseUriCombo, "ENT_URI", 3);
        
        workstationUriCombo = new JComboBox();
        workstationUriCombo.setEditable(true);
        getItemsFromPrefs(workstationUriCombo, "WKS_URI", 3);
        
        GridBagConstraints uriCon = new GridBagConstraints();
		uriCon.fill = GridBagConstraints.HORIZONTAL;
        uriCon.anchor = GridBagConstraints.LINE_START;
		uriCon.gridx = 0;
		uriCon.gridy = 0;
		uriCon.gridwidth = 3;
		uriCon.insets = new Insets(0,0,5,0);
		uriTabPane = new JTabbedPane();
		uriTabPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
		uriTabPane.addTab("Enterprise", enterpriseUriCombo);
		uriTabPane.addTab("Workstation", workstationUriCombo);
		uriTabPane.setSelectedIndex(prefs.getInt("TAB", 0));
		uriTabPane.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                if(uriTabPane.getSelectedComponent() == enterpriseUriCombo) {
                    otpCheck.setEnabled(true);
                } else {
                    otpCheck.setEnabled(false);
                    otpCheck.setSelected(false);
                }
            }
        });
        pane.add(uriTabPane, uriCon);
        
        GridBagConstraints serviceCon = new GridBagConstraints();
        serviceCon.fill = GridBagConstraints.HORIZONTAL;
        serviceCon.gridx = 0;
        serviceCon.gridy = 1;
        serviceCombo = new JComboBox();
        pane.add(serviceCombo, serviceCon);
        
        GridBagConstraints keyCon = new GridBagConstraints();
        keyCon.fill = GridBagConstraints.HORIZONTAL;
        keyCon.anchor = GridBagConstraints.LINE_START;
        keyCon.gridx = 1;
        keyCon.gridy = 1;
        keyCon.gridwidth = 2;
        keyCon.insets = new Insets(0,5,0,0);
        keyCombo = new JComboBox();
        keyCombo.setEditable(true);
        getItemsFromPrefs(keyCombo, "KEY", 3);
        pane.add(keyCombo, keyCon);
        
        GridBagConstraints connectCon = new GridBagConstraints();
        connectCon.anchor = GridBagConstraints.LINE_END;
        connectCon.gridx = 3;
        connectCon.gridy = 0;
        connectCon.insets = new Insets(15,20,0,0);
        connectButton = new JButton();
        connectButton.setText("Connect");
        connectButton.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		connect();
        	}
        });
        frame.getRootPane().setDefaultButton(connectButton);
        pane.add(connectButton, connectCon);
        
        GridBagConstraints disconnectCon = new GridBagConstraints();
        disconnectCon.anchor = GridBagConstraints.LINE_START;
        disconnectCon.gridx = 4;
        disconnectCon.gridy = 0;
        disconnectCon.insets = new Insets(15,15,0,0);
        disconnectButton = new JButton();
        disconnectButton.setText("Disconnect");
        disconnectButton.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		disconnect();
        	}
        });
        pane.add(disconnectButton, disconnectCon);
        
        GridBagConstraints requestCon = new GridBagConstraints();
        requestCon.anchor = GridBagConstraints.LINE_END;
        requestCon.gridx = 3;
        requestCon.gridy = 1;
        requestCon.insets = new Insets(0,20,0,0);
        requestButton = new JButton();
        requestButton.setText("Request");
        requestButton.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		request();
        	}
        });
        pane.add(requestButton, requestCon);
        
        GridBagConstraints cancelCon = new GridBagConstraints();
        cancelCon.anchor = GridBagConstraints.LINE_START;
        cancelCon.gridx = 4;
        cancelCon.gridy = 1;
        cancelCon.ipadx = 25;
        cancelCon.fill = GridBagConstraints.HORIZONTAL;
        cancelCon.insets = new Insets(0,15,0,0);
        cancelButton = new JButton();
        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		cancel();
        	}
        });
        pane.add(cancelButton, cancelCon);
        
        GridBagConstraints totalViewCon = new GridBagConstraints();
        totalViewCon.anchor = GridBagConstraints.FIRST_LINE_START;
        totalViewCon.fill = GridBagConstraints.HORIZONTAL;
        totalViewCon.gridx = 5;
        totalViewCon.gridy = 1;
        totalViewCheck = new JCheckBox("Nasdaq TotalView", false);
        pane.add(totalViewCheck, totalViewCon);

        GridBagConstraints otpCon = new GridBagConstraints();
        otpCon.anchor = GridBagConstraints.FIRST_LINE_START;
        otpCon.fill = GridBagConstraints.HORIZONTAL;
        otpCon.gridx = 5;
        otpCon.gridy = 0;
        otpCon.insets = new Insets(20,0,0,0);
        otpCheck = new JCheckBox("Authenticate with OTP", false);
        pane.add(otpCheck, otpCon);

        bidTable = new SwingL2Table("Bid");
		bidTable.setDefaultRenderer(String.class, new BidCellRenderer());

        askTable = new SwingL2Table("Ask");
		askTable.setDefaultRenderer(String.class, new AskCellRenderer());

        JScrollPane bidScrollPane = new JScrollPane(bidTable);
        JScrollPane askScrollPane = new JScrollPane(askTable);
        
        GridBagConstraints bidCon = new GridBagConstraints();
        bidCon.fill = GridBagConstraints.BOTH;
        bidCon.gridx = 0;
        bidCon.gridy = 2;
        bidCon.ipady = 180;
        bidCon.gridwidth = 6;
        bidCon.weightx = 0.5;
        bidCon.weighty = 0.1;
        bidCon.insets = new Insets(10,0,0,0);
        pane.add(bidScrollPane, bidCon);
        
        GridBagConstraints askCon = new GridBagConstraints();
        askCon.fill = GridBagConstraints.BOTH;
        askCon.gridx = 6;
        askCon.gridy = 2;
        askCon.ipady = 180;
        askCon.gridwidth = 6;
        askCon.weightx = 0.5;
        askCon.weighty = 0.1;
        askCon.insets = new Insets(10,0,0,0);
        pane.add(askScrollPane, askCon);
        
        GridBagConstraints statusCon = new GridBagConstraints();
        statusCon.fill = GridBagConstraints.HORIZONTAL;
        statusCon.gridx = 0;
        statusCon.gridy = 3;
        statusCon.gridwidth = 5;
        statusLabel = new JLabel();
        pane.add(statusLabel, statusCon);
        
        GridBagConstraints updatesCon = new GridBagConstraints();
        updatesCon.fill = GridBagConstraints.HORIZONTAL;
        updatesCon.gridx = 0;
        updatesCon.gridy = 4;
        updatesCon.gridwidth = 3;
        updatesLabel = new JLabel();
        pane.add(updatesLabel, updatesCon);
        
        GridBagConstraints closeCon = new GridBagConstraints();
        closeCon.anchor = GridBagConstraints.LINE_END;
        closeCon.insets = new Insets(0,0,10,0);
        closeCon.gridx = 11;
        closeCon.gridy = 4;
        closeButton = new JButton();
        closeButton.setText("Close");
        closeButton.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		frame.dispose();
        	}
        });
        
        resetGUIState();
        pane.add(closeButton, closeCon);
	}
	
	private static void addItemToCombo(JComboBox combo, String item) {
		for (int i = 0; i < combo.getItemCount(); ++i) {
			if (combo.getItemAt(i).equals(item)) {
				combo.removeItemAt(i);
				break;
			}
		}
		combo.insertItemAt(item, 0);
		combo.setSelectedIndex(0);
	}
	
	private void getItemsFromPrefs(JComboBox combo, String key, int numItems) {
		for (int i = 0; i < numItems; ++i) {
			String item = prefs.get(key + "_" + i, "");
			if (!item.isEmpty())
				combo.addItem(prefs.get(key + "_" + i, ""));
		}
	}
	
	private void putItemsInPrefs(JComboBox combo, String key, int numItems) {
		for (int i = 0; (i < numItems && i < combo.getItemCount()); ++i) {
			prefs.put(key + "_" + i, (String) combo.getItemAt(i));
		}
	}

	private void createAndShowGUI(String[] args) {	
		// Create and set up the window.
        frame = new JFrame("SwingL2");
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        frame.addWindowListener(new WindowAdapter() {
        	@Override
			public void windowClosed(WindowEvent we) {
        		if (consumer != null)
        			consumer.disconnect();
        		putItemsInPrefs(enterpriseUriCombo, "ENT_URI", 3);
        		putItemsInPrefs(workstationUriCombo, "WKS_URI", 3);
        		putItemsInPrefs(keyCombo, "KEY", 3);
        		prefs.putInt("TAB", uriTabPane.getSelectedIndex());
        	}
        });
        addComponentsToPane();
        
        // Display the window.
        frame.pack();
        frame.setSize(1000, 800);
        frame.setVisible(true);
        
        try {
			FileHandler fh = new FileHandler("SwingL2.log");
			logger.addHandler(fh);
			Logger.getLogger("com.factset.rt").addHandler(fh);
		} catch (IOException e) {
			logger.log(Level.WARNING, "Error opening log file", e);
		}
    }

	public static void main(final String[] args) {
		// Schedule a job for the event-dispatching thread:
        // creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
            	SwingL2 main = new SwingL2();
                main.createAndShowGUI(args);
            }
        });
	}
}
